/*****
*
*   Node.js
*
*   written by Kevin Lindsey
*   copyright 2001-2002, Kevin Lindsey
*
*   Based on Sven Moen's article "Drawing Dynamic Trees"
*
*****/

/*****
*
*   main constructor
*
*****/
function Node(name) {
    this.name   = name;
    this.parent = null;
    this.children = new Array();
    this.width    = 60;
    this.height   = 15;
    this.position = new Point2D(0, 0);
    this.offset   = new Point2D(0, 0);
    this.contour  = new Contour();
    this.pad_x    = 50;
    this.pad_y    = 1;
    this.visible  = true;
}


/*****
*
*   appendChild
*
*****/
Node.prototype.appendChild = function(child) {
    child.parent = this;
    this.children[this.children.length] = child;
};


/*****
*
*   Graph Drawing Routines
*
*****/

/*****
*
*   layout
*
*****/
Node.prototype.layout = function() {
    var length = this.children.length;
    
    for ( var i = 0; i < length; i++ ) {
        this.children[i].layout();
    }
    
    if ( length > 0 ) {
        this.attachParent( this.join() );
    } else {
        this.layoutLeaf();
    }
};


/*****
*
*   attachParent
*
*****/
Node.prototype.attachParent = function(height) {
    var x  = this.pad_x;
    var y2 = (height - this.height)/2 - this.pad_y;
    var y1 = y2 + this.height + 2 * this.pad_y - height;

    this.children[0].offset.setXY(x+this.width, y1);

    this.contour.upper_head = new ContourSegment(
        this.width, 0,
        new ContourSegment(
            x, y1,
            this.contour.upper_head
        )
    );
    this.contour.lower_head = new ContourSegment(
        this.width, 0,
        new ContourSegment(
            x, y2,
            this.contour.lower_head
        )
    );      
};


/*****
*
*   layoutLeaf
*
*****/
Node.prototype.layoutLeaf = function() {
    var contour = this.contour;

    contour.upper_tail = new ContourSegment(this.width + 2 * this.pad_x, 0, null);
    contour.upper_head = contour.upper_tail;
    contour.lower_tail = new ContourSegment(0, -this.height - 2 * this.pad_y, null);
    contour.lower_head = new ContourSegment(this.width + 2 * this.pad_x, 0, contour.lower_tail);
};


/*****
*
*   join
*
*****/
Node.prototype.join = function() {
    var child   = this.children[0];
    var contour = this.contour = child.contour.clone();
    var height  = child.height + 2*child.pad_y;
    var sum     = height;

    for (var i = 1; i < this.children.length; i++) {
        var child = this.children[i];
        var merged_height = contour.merge(child.contour);

        child.offset.setXY(0, height + merged_height);

        height = child.height + 2*child.pad_y;
        sum += height + merged_height;
    }

    return sum;
};


/*****
*
*   Tree Drawing Methods
*
*****/

/*****
*
*   plantTree
*
*****/
Node.prototype.plantTree = function(dx, dy) {
    var last_x = 0;
    var last_y = 0;

    this.position.setXY(dx + this.offset.x, dy + this.offset.y);

    for (var i = 0; i < this.children.length; i++) {
        var child = this.children[i];

        child.plantTree(this.position.x + last_x, this.position.y + last_y);
        if ( i == 0 ) {
            last_x = child.offset.x;
        }
        last_y += child.offset.y;
    }
};


/*****
*
*   realize
*
*****/
Node.prototype.realize = function(svgParentNode) {
    var x           = this.position.x;
    var y           = this.position.y;
    var half_width  = this.width / 2;
    var half_height = this.height / 2;
    
    // Draw lines to all child nodes
    for (var i = 0; i < this.children.length; i++) {
        var child = this.children[i];

        if ( child.visible ) {
            var line  = svgDocument.createElement("line");
            
            line.setAttribute("x1", x + half_width);
            line.setAttribute("y1", y);
            line.setAttribute("x2", child.position.x - child.width / 2);
            line.setAttribute("y2", child.position.y);
            line.setAttribute("stroke", "black");
            line.setAttribute("stroke-width", "2");
            line.setAttribute("stroke-linecap", "round");

            svgParentNode.appendChild(line);

            child.realize(svgParentNode);
        }
    }

    // Create graphic for this node
    var rect  = svgDocument.createElement("rect");
    var text  = svgDocument.createElement("text");
    var tspan = svgDocument.createElement("tspan");
    var tnode = svgDocument.createTextNode(this.name);

    rect.setAttribute("x", x - half_width);
    rect.setAttribute("y", y - half_height);
    rect.setAttribute("width", this.width);
    rect.setAttribute("height", this.height);
    rect.setAttribute("rx", "3");
    rect.setAttribute("ry", "3");
    rect.setAttribute(
        "fill",
        (this.color) ? this.color : "grey"
    );
    this.rect = rect;

    text.setAttribute("x", x);
    text.setAttribute("y", y);
    text.setAttribute("fill", "white");
    text.setAttribute("text-anchor", "middle");
    this.text = text;

    tspan.setAttribute("dy", "0.33em");

    tspan.appendChild(tnode);
    text.appendChild(tspan);
    svgParentNode.appendChild(rect);
    svgParentNode.appendChild(text);
};
